@@ -1,5 +1,5 @@
+use crate::frame::{util, Error, Frame, Head, Kind, StreamId};
 use bytes::{Buf, BufMut, Bytes};
-use frame::{util, Error, Frame, Head, Kind, StreamId};
 
 use std::fmt;
 
@@ -16,7 +16,7 @@ pub struct Data<T = Bytes> {
     pad_len: Option<u8>,
 }
 
-#[derive(Copy, Clone, Eq, PartialEq)]
+#[derive(Copy, Clone, Default, Eq, PartialEq)]
 struct DataFlags(u8);
 
 const END_STREAM: u8 = 0x1;
@@ -29,14 +29,14 @@ impl<T> Data<T> {
         assert!(!stream_id.is_zero());
 
         Data {
-            stream_id: stream_id,
+            stream_id,
             data: payload,
             flags: DataFlags::default(),
             pad_len: None,
         }
     }
 
-    /// Returns the stream identifer that this frame is associated with.
+    /// Returns the stream identifier that this frame is associated with.
     ///
     /// This cannot be a zero stream identifier.
     pub fn stream_id(&self) -> StreamId {
@@ -63,7 +63,7 @@ impl<T> Data<T> {
         }
     }
 
-    /// Returns whther the `PADDED` flag is set on this frame.
+    /// Returns whether the `PADDED` flag is set on this frame.
     #[cfg(feature = "unstable")]
     pub fn is_padded(&self) -> bool {
         self.flags.is_padded()
@@ -135,10 +135,24 @@ impl Data<Bytes> {
         Ok(Data {
             stream_id: head.stream_id(),
             data: payload,
-            flags: flags,
-            pad_len: pad_len,
+            flags,
+            pad_len,
         })
     }
+
+    pub(crate) fn flow_controlled_len(&self) -> usize {
+        if let Some(pad_len) = self.pad_len {
+            // if PADDED, pad length field counts too (the + 1)
+            self.data.len() + usize::from(pad_len) + 1
+        } else {
+            self.data.len()
+        }
+    }
+
+    /// If this frame is PADDED, it returns the pad len + 1 (length field).
+    pub(crate) fn padded_len(&self) -> Option<u8> {
+        self.pad_len.map(|n| n + 1)
+    }
 }
 
 impl<T: Buf> Data<T> {
@@ -148,7 +162,7 @@ impl<T: Buf> Data<T> {
     ///
     /// Panics if `dst` cannot contain the data frame.
     pub(crate) fn encode_chunk<U: BufMut>(&mut self, dst: &mut U) {
-        let len = self.data.remaining() as usize;
+        let len = self.data.remaining();
 
         assert!(dst.remaining_mut() >= len);
 
@@ -211,12 +225,6 @@ impl DataFlags {
     }
 }
 
-impl Default for DataFlags {
-    fn default() -> Self {
-        DataFlags(0)
-    }
-}
-
 impl From<DataFlags> for u8 {
     fn from(src: DataFlags) -> u8 {
         src.0
