@@ -1,7 +1,6 @@
-use codec::RecvError;
-use error::Reason;
-use frame::{Pseudo, StreamId};
-use proto::Open;
+use crate::error::Reason;
+use crate::frame::{Pseudo, StreamId};
+use crate::proto::{Error, Open};
 
 use http::{HeaderMap, Request, Response};
 
@@ -11,19 +10,24 @@ use std::fmt;
 pub(crate) trait Peer {
     /// Message type polled from the transport
     type Poll: fmt::Debug;
+    const NAME: &'static str;
 
-    fn dyn() -> Dyn;
+    fn r#dyn() -> Dyn;
 
-    fn is_server() -> bool;
+    //fn is_server() -> bool;
 
     fn convert_poll_message(
-        pseudo: Pseudo, fields: HeaderMap, stream_id: StreamId
-    ) -> Result<Self::Poll, RecvError>;
+        pseudo: Pseudo,
+        fields: HeaderMap,
+        stream_id: StreamId,
+    ) -> Result<Self::Poll, Error>;
 
+    /*
     fn is_local_init(id: StreamId) -> bool {
         assert!(!id.is_zero());
         Self::is_server() == id.is_server_initiated()
     }
+    */
 }
 
 /// A dynamic representation of `Peer`.
@@ -54,24 +58,27 @@ impl Dyn {
     }
 
     pub fn convert_poll_message(
-        &self, pseudo: Pseudo, fields: HeaderMap, stream_id: StreamId
-    ) -> Result<PollMessage, RecvError> {
+        &self,
+        pseudo: Pseudo,
+        fields: HeaderMap,
+        stream_id: StreamId,
+    ) -> Result<PollMessage, Error> {
         if self.is_server() {
-            ::server::Peer::convert_poll_message(pseudo, fields, stream_id)
+            crate::server::Peer::convert_poll_message(pseudo, fields, stream_id)
                 .map(PollMessage::Server)
         } else {
-            ::client::Peer::convert_poll_message(pseudo, fields, stream_id)
+            crate::client::Peer::convert_poll_message(pseudo, fields, stream_id)
                 .map(PollMessage::Client)
         }
     }
 
     /// Returns true if the remote peer can initiate a stream with the given ID.
-    pub fn ensure_can_open(&self, id: StreamId, mode: Open) -> Result<(), RecvError> {
+    pub fn ensure_can_open(&self, id: StreamId, mode: Open) -> Result<(), Error> {
         if self.is_server() {
             // Ensure that the ID is a valid client initiated ID
             if mode.is_push_promise() || !id.is_client_initiated() {
                 proto_err!(conn: "cannot open stream {:?} - not client initiated", id);
-                return Err(RecvError::Connection(Reason::PROTOCOL_ERROR));
+                return Err(Error::library_go_away(Reason::PROTOCOL_ERROR));
             }
 
             Ok(())
@@ -79,7 +86,7 @@ impl Dyn {
             // Ensure that the ID is a valid server initiated ID
             if !mode.is_push_promise() || !id.is_server_initiated() {
                 proto_err!(conn: "cannot open stream {:?} - not server initiated", id);
-                return Err(RecvError::Connection(Reason::PROTOCOL_ERROR));
+                return Err(Error::library_go_away(Reason::PROTOCOL_ERROR));
             }
 
             Ok(())
